/*
* IMPACT
 *
 * This class simulates an impact triggered by mouse click
 *
 * @name Impact
 * @author Sebastian Pape <sebastian.pape@zhdk.ch>
 * @version 15.12.2010
 */

class Impact {

  /*
  * VARIABLES */
  float    startMillis;
  float    startX;
  float    startY;
  float    diameter;
  float    maxAnimationLength = 2000;
  float    currentAnimationLength = 0;
  float    normedAnimationTime;
  float    impactRadius = 0;
  float    collisionStartDiameter;
  Boolean  limiter = true;
  Boolean  focusedImpact = false;

  /*
  * CONSTRUCTOR */
  Impact(float _x, float _y) {
    startMillis = millis();
    startX = _x;
    startY = _y;
  }

  /*
  * SIMULATE */
  void simulate() {
    // CURRENT ANIMATION LENGTH
    currentAnimationLength = millis() - startMillis;

    // IF HIGHER THAN MAX SET MAX
    if(currentAnimationLength > maxAnimationLength) {
      currentAnimationLength = maxAnimationLength;
    }

    // NORM ANIMATION TIME
    normedAnimationTime = norm(currentAnimationLength, 0, maxAnimationLength);

    // CHECK DISTANCE TO SIGNS TO SIMULATE COLLISION
    for(int i = 0; i < permutationManager.signs.size(); i++) {  
      // GET SIGN
      Sign _sign = (Sign)permutationManager.signs.get(i);

      // DISTANCE
      float _distance = dist(_sign.x, _sign.y, startX, startY);

      // IMPACT RADIUS
      impactRadius = screen.width * 1.2 * normedAnimationTime;

      // COLLISON
      if(focusedImpact) {
        
        if(_distance < _sign.diameter/2) {
          // IF LIMITER IS ACTIVATED CHECK THAT collisionTransformer do not become to big
          if(limiter && _sign.collisionTransformer >= 1) {
          } else {
            _sign.collisionTransformer += normedAnimationTime;
          }
        }
        
      } else {
      
        if(_distance - _sign.diameter < impactRadius && !(impactRadius > _distance + _sign.diameter)) {
          // IF LIMITER IS ACTIVATED CHECK THAT collisionTransformer do not become to big
          if(limiter && _sign.collisionTransformer >= 1) {
          } else {
            _sign.collisionTransformer += .2;
          }
        }
        
      }
    }
  }
}

