/*
* SIGNS
*
* This class manages the sign itselfes and draw it with given parameters
*
* @name Sign
* @author Sebastian Pape <sebastian.pape@zhdk.ch>
* @version 09.12.2010
*/

class Sign {
  
  /*
  * VARIABLES */
  int    x;
  int    y;
  int    type;
  int    diameter;
  
  float  collisionTransformer = 0;
  float  soundTransformer = 0.1;
  
  /*
  * CONSTRUCTOR */
  Sign(int _x, int _y, int _diameter, int _type) {
    x = _x;
    y = _y;
    diameter = _diameter;
    type = _type;
  }
  
  /*
  * GET MOUSE DEGREE */
  float getMouseDegree(float _x, float _y) {
    float mouseDegree = degrees(atan2(mouseY - _y, mouseX - _x));
  
    if (mouseDegree < 0) {
      // negative
      if(mouseDegree >= -90) {
        mouseDegree = (mouseDegree+90);
      } 
      else {
        mouseDegree = (mouseDegree+450);
      }
    } 
    else {
      mouseDegree = (mouseDegree+90);
    }
  
    return mouseDegree;
  }
  
  /*
  * GET MOUSE DISTANCE */
  float getMouseDistance(float _x, float _y) {
    float mouseDistance = dist(_x, _y, mouseX, mouseY);;
    return mouseDistance;
  }
  
  /*
  * DRAW */
  void draw() {
    // BRING DOWN COLISION TRANSFORMER
    bringDownCollisionTransformer();
    
    // BRING DOWN COLISION TRANSFORMER
    bringDownSoundTransformer();
    
    // MOUSEDISTANCE
    float _normalizedMouseDistance = norm(getMouseDistance(x,y),0, screen.width);
  
    // DRAW
    switch(type) {
      case 1:
        drawSignType1(_normalizedMouseDistance);
      break;
      case 2:
        drawSignType2(_normalizedMouseDistance);
      break;
      case 3:
        drawSignType3(_normalizedMouseDistance);
      break;
      default: 
        drawSignType1(_normalizedMouseDistance);
      break; 
    }

  }
  
  /* 
  * DRAW SIGN TYPE 1 */
  void drawSignType1(float _normalizedMouseDistance){
    // TRANSLATE
    pushMatrix();
      translate(x,y);
      
      // ROTATE MOUSE DEGREE
      pushMatrix();
        rotate(radians(getMouseDegree(x,y)));
        
          // COLLISION TRANSFORM
          pushMatrix();
             scale(.5 + collisionTransformer/1.5);
             
            // SOUND TRANSFORMATION
            pushMatrix();
              scale(1.5*soundTransformer);
        
              /* CIRCLE 1 BLACK */
              stroke(0);
              fill(0);
              ellipse(
                      0,
                      0,
                      diameter*.95,
                      diameter*.95
              );
              
              /* CIRCLE 2 WHITE */
              stroke(0);
              fill(255);
              ellipse(
                      0,
                      -_normalizedMouseDistance*diameter*.01,
                      diameter*.85 - diameter*_normalizedMouseDistance*.05,
                      diameter*.85 - diameter*_normalizedMouseDistance*.05
              );
            
            popMatrix();
      
               /* CIRCLE 3 BLACK */
              stroke(0);
              fill(0);
              ellipse(
                      0,
                      -_normalizedMouseDistance*diameter*.045,
                      diameter*.33 + diameter*_normalizedMouseDistance*.53,
                      diameter*.33 + diameter*_normalizedMouseDistance*.53
               );

                
               /* CIRCLE 4 WHITE */
              stroke(0);
              fill(255);
              ellipse(
                      0,
                      -_normalizedMouseDistance*diameter*.2,
                      diameter*.40 + diameter*_normalizedMouseDistance*.1,
                      diameter*.40 + diameter*_normalizedMouseDistance*.1
              );
              
               /* CIRCLE 5 BLACK */
               // SOUND TRANSFORMATION
              pushMatrix();
                scale(1+soundTransformer);
                stroke(0);
                fill(0);
                ellipse(
                        0,
                        -_normalizedMouseDistance*diameter*.28,
                        diameter*.15 + diameter*_normalizedMouseDistance*.2,
                        diameter*.15 + diameter*_normalizedMouseDistance*.2
                );
              popMatrix(); // SOUND TRANSFORM
          
          popMatrix();// COLLISION TRANSFORM
      popMatrix();// ROTATE
    popMatrix();// TRANSLATE
  }
  
  /* 
  * DRAW SIGN TYPE 2 */
  void drawSignType2(float _normalizedMouseDistance){
    // TRANSLATE
    pushMatrix();
      translate(x,y);
        
  /*
  * SQUARE 1 BLACK */

        // ROTATE MOUSEDEGREE
        pushMatrix();
          rotate(radians(getMouseDegree(x,y)));
        
          // COLLISION TRANSFORM
          pushMatrix();
             scale(1 + collisionTransformer/2);
          
              stroke(0);
              fill(0);
              rect(
                   0,
                   0,
                   diameter*.5,
                   diameter*.5
              );
            popMatrix();// COLLISION TRANSFORM
        popMatrix();// ROTATE MOUSEDEGREE

    /*
    * SQUARE 2 WHITE */

        // ROTATE MOUSEDEGREE
        pushMatrix();
          rotate(radians(getMouseDegree(x,y)));
        
          // COLLISION TRANSFORM
          pushMatrix();
             scale(1 + collisionTransformer/2);
             
             // SOUND TRANSFORM
              pushMatrix();
                  if(soundTransformer < 0.08) {
                    soundTransformer = 0.01;
                     fill(0);
                  } else {
                     fill(255);
                  }
                  scale(soundTransformer*1.5);
          
                stroke(0);
                rect(
                     0,
                     0,
                     diameter*.6,
                     diameter*.6
                );
              popMatrix();// SOUND TRANSFORM
            popMatrix();// COLLISION TRANSFORM
        popMatrix();// ROTATE MOUSEDEGREE

      
    popMatrix();// TRANSLATE
  }
  
   /* 
  * DRAW SIGN TYPE 3 */
  void drawSignType3(float _normalizedMouseDistance){
    // TRANSLATE
    pushMatrix();
      translate(x,y);
        
  /*
  * TRIANGLE 1 BLACK */

        // ROTATE MOUSEDEGREE
        pushMatrix();
           rotate(radians(90+getMouseDegree(x,y)));
        
          // COLLISION TRANSFORM
          pushMatrix();
             scale(.55 + collisionTransformer*.5);
          
              stroke(255);
              strokeWeight(8);
              fill(0);
              triangle(
                     0,
                     0,
                     diameter*.5,
                     diameter,
                     diameter,
                     0
              );
            popMatrix();// COLLISION TRANSFORM
        popMatrix();// ROTATE MOUSEDEGREE
        
         /*
  * TRIANGLE 2 BLACK */

        // ROTATE MOUSEDEGREE
        pushMatrix();
           rotate(radians(90+getMouseDegree(x,y)));
        
          // COLLISION TRANSFORM
          pushMatrix();
             scale(.5 - collisionTransformer*.5);
             
             // SOUND TRANSFORM
             pushMatrix();
               scale(1.3-soundTransformer*.8);
          
                stroke(0);
                fill(0);
                triangle(
                       -diameter,
                       -diameter,
                       -diameter*.5,
                       -diameter,
                       -diameter*.7,
                       -diameter*.5
                );
                
                popMatrix();// SOUND TRANSFORM
            popMatrix();// COLLISION TRANSFORM
        popMatrix();// ROTATE MOUSEDEGREE

    /*
    * TRIANGLE WHITE */

        // ROTATE MOUSEDEGREE
        pushMatrix();
          rotate(radians(90+getMouseDegree(x,y)));
        
          // COLLISION TRANSFORM
          pushMatrix();
             scale(1 + collisionTransformer/2);
             
             // SOUND TRANSFORM
              pushMatrix();
                  if(soundTransformer < 0.08) {
                    soundTransformer = 0.01;
                     fill(0);
                  } else {
                     fill(255);
                  }
                  scale(soundTransformer*.8);
          
                stroke(0);
                triangle(
                     0,
                     0,
                     diameter*.5,
                     diameter,
                     diameter,
                     0
                );
              popMatrix();// SOUND TRANSFORM
            popMatrix();// COLLISION TRANSFORM
        popMatrix();// ROTATE MOUSEDEGRE
      
    popMatrix();// TRANSLATE
  }
  
  /* 
  * BRING DOWN COLLISION TRANSFORMER */
  void bringDownCollisionTransformer() {
      collisionTransformer -= .1;
      
      if(collisionTransformer < 0) {
         collisionTransformer = 0; 
      }
  }
  
  /* 
  * BRING DOWN SOUND TRANSFORMER */
  void bringDownSoundTransformer() {
      soundTransformer -= .1;
      
      if(soundTransformer <= 0) {
       soundTransformer = .05; 
      }
  }
  
}
