/*
* VJ TOOL
*
* This programm creates a permutation with signs and provides multiple functionalities to
* let the user interact with the permutation and provides sound input
*
* @name Interactive Permutation
* @author Sebastian Pape <sebastian.pape@zhdk.ch>
* @version 09.12.2010
*/

/*
* IMPORTS */
import ddf.minim.*;
import ddf.minim.analysis.*;

/*
 * VARIABLES */
PermutationManager       permutationManager;
ImpactManager            impactManager;
Minim                    minim;
AudioPlayer              audioPlayer;
AudioInput               audioInput;
BeatDetect               beat;
Boolean                  beatDetection = false;
Boolean                  levelDetection = false;
                           
/*
* SETUP */
void setup() {
  // SIZE
  size(screen.width, screen.height);
  
  // SMOOTH
  smooth();
  
  // PERMUTATION MANAGER
  permutationManager = new PermutationManager();
  
  // SET UP PERMUTATION
  permutationManager.setUpPermutation(); 
  
  // IMPACT MANAGER
  impactManager = new ImpactManager();
  
  // MINIM
  minim = new Minim(this);
  audioPlayer = minim.loadFile("sounds/LK3D.mp3");
  audioInput = minim.getLineIn();
  audioPlayer.loop();
  beat = new BeatDetect();
}

/*
 * DRAW */
void draw()
{
  // BACKGROUND
  background(255);
  
  // NO MOUSE CURSOR
  noCursor();
  
  // DRAW SIGNS
  permutationManager.drawSigns();
  
  // SIMULATE IMPACTS
  impactManager.simulateAllImpacts();
  
  // DETECT BEAT
   beat.detect(audioPlayer.mix);
   if ( beat.isOnset() && beatDetection ) impactManager.newImpact(mouseX, mouseY);
   
   // DETECT VOLUME
   if(levelDetection) {
     for(int i = 0; i < permutationManager.signs.size(); i++) {  
       // GET SIGN
        Sign _sign = (Sign)permutationManager.signs.get(i);
        
        // SOUND TRANSFORMER
        if(norm((audioPlayer.left.level() + audioPlayer.right.level()), 0, 2) * 2 > _sign.soundTransformer) {
          _sign.soundTransformer = norm((audioPlayer.left.level() + audioPlayer.right.level()), 0, 2);
        }
     }
   }
}

/*
* KEY PRESSED */
void keyPressed() {
  switch(keyCode) {
    case UP:
      permutationManager.zoomIn();
      
    break; 
    case DOWN:
      permutationManager.zoomOut();
    break; 
  }
  switch(key) {
      case ' ':
        beatDetection = !beatDetection;
      break; 
      case 10:
        levelDetection = !levelDetection;
      break;
      case 'l':
        impactManager.limiter = !impactManager.limiter;
      break;
      case 'f':
        impactManager.focusedImpact = !impactManager.focusedImpact;
      break;
      case '1':
        setSignType(1);
      break;
      case '2':
        setSignType(2);
      break;
      case '3':
        setSignType(3);
      break;
      case '4':
        setSignType(4);
      break;
   }
 }
 
 /* 
 * SET SIGN TYPE */
 void setSignType(int _type) {
      permutationManager.currentSignType = _type;
      permutationManager.setUpPermutation();
 }
 
/*
* MOUSE PRESSED */
void mousePressed() {
  impactManager.newImpact(mouseX, mouseY);
}

/*
* STOP */
void stop()
{
  // always close Minim audio classes when you are finished with them
  audioPlayer.close();
  // always stop Minim before exiting
  minim.stop();
 
  super.stop();
}

